# -*- coding: utf-8 -*-

"""Main module."""
from bluepy.btle import Peripheral

from thingy52.services import EnvironmentService, MotionService, UserInterfaceService, SoundService
from thingy52.characteristics import thingy_chars_map

# Write this to CCCD to enable notifications
NOTIFY_ON = (1).to_bytes(2, byteorder='little')
NOTIFY_OFF = (0).to_bytes(2, byteorder='little')


class Thingy52(Peripheral):
    """
    Thingy:52 module. Instance the class and enable to get access to the Thingy:52 Sensors.
    The addr of your device has to be know, or can be found by using the hcitool command line
    tool, for example. Call "> sudo hcitool lescan" and your Thingy's address should show up.
    """

    handles = {}

    def __init__(self, addr):
        Peripheral.__init__(self, addr, addrType="random")

        # self.battery = BatterySensor(self)
        self.environment = EnvironmentService(self)
        self.ui = UserInterfaceService(self)
        self.motion = MotionService(self)
        self.sound = SoundService(self)

    """
    @property
    def services(self):
        service_mapping = {
            Nordic_UUID(ENVIRONMENT_SERVICE_UUID): EnvironmentService,
        }
        peripheral_services = super(Thingy52, self).services
        thingy_services = [service_mapping.get(s.uuid, s) for s in peripheral_services]
        return thingy_services
    """

    def toggle_notify(self, char, enable):
        thingy_char = thingy_chars_map.get(char.uuid, None)
        self.handles[char.getHandle()] = thingy_char
        self.writeCharacteristic(char.getHandle() + 1, NOTIFY_ON if enable else NOTIFY_OFF)

    def write(self, char, command):
        # Todo: Exception handling for char.uuid not found in dict
        thingy_char = thingy_chars_map[char.uuid]
        print(thingy_char)
        self.handles[char.getHandle()] = thingy_char
        self.writeCharacteristic(char.getHandle(), command)



"""PyAudio example: Record a few seconds of audio and save to a WAVE file."""
from bluepy.btle import DefaultDelegate
import pyaudio
import wave
import audioop
import binascii
from struct import Struct
import struct

CHUNK = 20
FORMAT = pyaudio.paInt16
CHANNELS = 1
# https://infocenter.nordicsemi.com/index.jsp?topic=%2Fcom.nordic.infocenter.rds%2Fdita%2Frds%2Fdesigns%2Fthingy%2Fintro%2Fkey_features.html
RATE = 16000
RECORD_SECONDS = 5
WAVE_OUTPUT_FILENAME = "output.wav"

# Intel ADPCM step variation table
INDEX_TABLE = [-1, -1, -1, -1, 2, 4, 6, 8, -1, -1, -1, -1, 2, 4, 6, 8, ]

# ADPCM step size table
STEP_SIZE_TABLE = [7, 8, 9, 10, 11, 12, 13, 14, 16, 17, 19, 21, 23, 25, 28, 31, 34, 37, 41, 45, 50, 55, 60, 66, 73, 80,
                   88, 97, 107, 118, 130, 143, 157, 173, 190, 209,
                   230, 253, 279, 307, 337, 371, 408, 449, 494, 544, 598, 658, 724, 796, 876, 963, 1060, 1166, 1282,
                   1411, 1552, 1707, 1878, 2066, 2272, 2499, 2749, 3024, 3327, 3660, 4026, 4428, 4871, 5358,
                   5894, 6484, 7132, 7845, 8630, 9493, 10442, 11487, 12635, 13899, 15289, 16818, 18500, 20350, 22385,
                   24623, 27086, 29794, 32767]


class RecordingDelegate(DefaultDelegate):
    def __init__(self, handles):
        DefaultDelegate.__init__(self)
        self.handles = handles

        self.p = pyaudio.PyAudio()

        self.stream = self.p.open(format=FORMAT,
                                  channels=CHANNELS,
                                  rate=RATE,
                                  input=True,
                                  frames_per_buffer=CHUNK)

        print("* recording")

        self.frames = []

        self.state = None

    def handleNotification(self, cHandle, data):
        pcm, self.state = audioop.adpcm2lin(data, 2, self.state)
        self.frames.append(pcm)

    def finish(self):
        print("* done recording")

        self.stream.stop_stream()
        self.stream.close()
        self.p.terminate()

        wf = wave.open(WAVE_OUTPUT_FILENAME, 'wb')
        wf.setnchannels(CHANNELS)
        wf.setsampwidth(self.p.get_sample_size(FORMAT))
        wf.setframerate(RATE)
        wf.writeframes(b''.join(self.frames))
        wf.close()


class RecordingDelegate3(DefaultDelegate):
    def __init__(self, handles):
        DefaultDelegate.__init__(self)
        self.handles = handles
        self.p = pyaudio.PyAudio()
        self.frames = []
        self.state = None

        self.adpcmfile = open("recording.adpcm", "wb")

        print("* recording")
        self.numframes = 0

    def handleNotification(self, cHandle, data):
        self.adpcmfile.write(data)
        self.numframes += 1

    def finish(self):
        self.adpcmfile.close()
        print("* done recording", self.numframes, "frames")

        with open('recording.adpcm', 'rb') as f:
            adpcm = f.read()
        pcm, _ = audioop.adpcm2lin(adpcm, 2, None)
        # data length is 20 bytes, pcm length is 80 bytes
        self.wavfile = wave.open("out2" + '.wav', 'wb')
        self.wavfile.setparams((2, 2, 16000, 0, 'NONE', 'NONE'))
        self.wavfile.close()


class RecordingDelegate4(DefaultDelegate):
    def __init__(self, handles):
        DefaultDelegate.__init__(self)
        self.handles = handles
        self.p = pyaudio.PyAudio()
        self.frames = []
        self.state = None

        self.stream = self.audio.open(
            format=pyaudio.paInt16,
            channels=1, rate=16000,
            output=True, stream_callback=self.on_audio_ready)

        self.wavfile = wave.open("out2" + '.wav', 'wb')
        self.wavfile.setnchannels(CHANNELS)
        self.wavfile.setsampwidth(self.p.get_sample_size(FORMAT))
        self.wavfile.setframerate(RATE)
        self.numframes = 0
        print("* recording")

    def handleNotification(self, cHandle, data):
        pcm = adpcm_decode(data)
        self.wavfile.writeframes(pcm)
        self.numframes += 1

    def finish(self):
        print("* done recording", self.numframes, "frames")
        self.wavfile.close()


if __name__ == "__main__":
    CHUNK = 1024
    FORMAT = pyaudio.paInt16
    CHANNELS = 2
    RATE = 44100
    RECORD_SECONDS = 5
    WAVE_OUTPUT_FILENAME = "output.wav"

    p = pyaudio.PyAudio()

    stream = p.open(format=FORMAT,
                    channels=CHANNELS,
                    rate=RATE,
                    input=True,
                    frames_per_buffer=CHUNK)

    print("* recording")

    frames = []

    for i in range(0, int(RATE / CHUNK * RECORD_SECONDS)):
        data = stream.read(CHUNK)
        frames.append(data)

    print("* done recording")

    stream.stop_stream()
    stream.close()
    p.terminate()

    wf = wave.open(WAVE_OUTPUT_FILENAME, 'wb')
    wf.setnchannels(CHANNELS)
    wf.setsampwidth(p.get_sample_size(FORMAT))
    wf.setframerate(RATE)
    wf.writeframes(b''.join(frames))
    wf.close()


def adpcm_decode(adpcm):
    """
    Attempt to replicate node implementation from
    https://github.com/NordicSemiconductor/Nordic-Thingy52-Nodejs/blob/master/examples/microphone.js
    :param adpcm: bytes array
    :return: pcm
    """
    s = Struct('< hb17s')
    (valuePredicted, index, data) = s.unpack(adpcm)

    # Allocate output buffer
    # pcm = new Buffer(adpcm.data.length*4)
    # pcm = []*len(adpcm)*4
    pcm = b''
    # The first 2 bytes of ADPCM frame are the predicted value
    # valuePredicted = adpcm.header.readInt16BE(0)
    # The 3rd byte is the index value
    # index = adpcm.header.readInt8(2)

    if index < 0:
        index = 0
    if index > 88:
        index = 88

    # diff #/* Current change to valuePredicted */
    bufferStep = False
    inputBuffer = 0
    delta = 0
    sign = 0
    step = STEP_SIZE_TABLE[index]
    _in = 0
    _out = 0
    # for (var _in = 0, _out = 0; _in < adpcm.data.length; _out += 2) {
    # for _out in range(0, adpcm.data.length, step=2):
    # for _out in range(0, len(data), 2):
    while _in < len(adpcm):

        # Step 1 - get the delta value */
        if bufferStep:
            delta = inputBuffer & 0x0F
            _in += 1
        else:
            # inputBuffer = adpcm.data.readInt8(_in)
            inputBuffer = adpcm[_in]
            delta = (inputBuffer >> 4) & 0x0F
        bufferStep = not bufferStep

        # /* Step 2 - Find new index value (for later) */
        index += INDEX_TABLE[delta]
        if index < 0:
            index = 0
        if index > 88:
            index = 88

        # /* Step 3 - Separate sign and magnitude */
        sign = delta & 8
        delta = delta & 7

        # /* Step 4 - Compute difference and new predicted value */
        diff = step >> 3
        if (delta & 4) > 0:
            diff += step
        if (delta & 2) > 0:
            diff += step >> 1
        if (delta & 1) > 0:
            diff += step >> 2

        if sign > 0:
            valuePredicted -= diff
        else:
            valuePredicted += diff

        # /* Step 5 - clamp output value */
        if valuePredicted > 32767:
            valuePredicted = 32767
        elif valuePredicted < -32768:
            valuePredicted = -32768

        # /* Step 6 - Update step value */
        step = STEP_SIZE_TABLE[index]

        # /* Step 7 - Output value */
        # pcm.writeInt16LE(valuePredicted,  _out)
        pcm += struct.pack('< h', valuePredicted)

        _out += 2

    return pcm



# -*- coding: utf-8 -*-

"""Console script for gcloud_cli."""

import click
import atexit

from . import __version__
from . import thingy52
from thingy52 import delegates
from thingy52.audio import RecordingDelegate4
from thingy52.services import Color
import time

CONTEXT_SETTINGS = dict(
    obj={},
)


@click.group(context_settings=CONTEXT_SETTINGS)
@click.option('--debug/--no-debug', default=False)
@click.argument('address')
@click.pass_context
def main(ctx, debug, address):
    """Console script for gcloud_cli."""
    ctx.obj['address'] = address


@main.command()
def version():
    """Display the current version."""
    click.echo(__version__)


@click.command()
@click.pass_context
def list_services(ctx):
    """Console script for gcloud_cli."""
    pass


@main.command()
@click.pass_context
def list_services(ctx):
    """
    List services.
    """
    address = ctx.obj.get("address")
    t = thingy52.Thingy52(address)
    atexit.register(t.disconnect)


@main.command()
@click.pass_context
def list_characteristics(ctx):
    """
    List characteristics.
    """
    address = ctx.obj.get("address")
    t = thingy52.Thingy52(address)
    atexit.register(t.disconnect)


@main.command()
@click.argument('feature')
@click.option('--enable', default=True)
@click.pass_context
def motion(ctx, feature, enable):
    """
    Enable/Disable motion characteristics.
    """
    address = ctx.obj.get("address")
    print(address)

    t = thingy52.Thingy52(address)
    atexit.register(t.disconnect)
    t.setDelegate(delegates.ThingyCharDelegate(t.handles))
    t.motion.toggle_notifications(characteristic=feature, enable=enable)
    while True:
        t.waitForNotifications(1.0)
        print("Waiting...")

@main.command()
@click.argument('feature')
@click.option('--enable', default=True)
@click.pass_context
def sound(ctx, feature, enable):
    """
    """
    address = ctx.obj.get("address")
    print(address)

    t = thingy52.Thingy52(address)
    atexit.register(t.disconnect)
    t.setDelegate(delegates.ThingyCharDelegate(t.handles))
    t.sound.toggle_notifications(characteristic=feature, enable=enable)
    while True:
        t.waitForNotifications(1.0)
        print("Waiting...")

@main.command()
@click.pass_context
def record(ctx):
    """
    """
    address = ctx.obj.get("address")

    t = thingy52.Thingy52(address)
    # Set mic mode to ADPCM
    t.sound.activate_speaker_stream(1, 1)

    atexit.register(t.disconnect)
    rec = RecordingDelegate4(t.handles)
    t.setDelegate(rec)
    atexit.register(rec.finish)
    t.sound.toggle_notifications(characteristic="microphone", enable=True)
    now = time.time()
    while time.time() - now < 3:
        t.waitForNotifications(1.0)

@main.command()
@click.pass_context
def speaker_stream(ctx):
    """
    """
    address = ctx.obj.get("address")

    t = thingy52.Thingy52(address)
    atexit.register(t.disconnect)

    t.sound.activate_speaker_stream(2, 1)
    while True:
        print("k")
        t.sound.stream_speaker()


@main.command()
@click.pass_context
def speaker_frequency(ctx):
    """
    """
    address = ctx.obj.get("address")

    t = thingy52.Thingy52(address)
    atexit.register(t.disconnect)
    t.sound.activate_speaker_stream(1, 1)
    t.sound.stream_frequency(10500, 500, 50)


@main.command()
@click.pass_context
def demo_led(ctx):
    """
    Enable/Disable motion characteristics.
    """
    address = ctx.obj.get("address")

    t = thingy52.Thingy52(address)
    atexit.register(t.disconnect)
    d = delegates.Demo1Delegate(t, t.handles)
    t.setDelegate(d)
    t.ui.toggle_notifications(characteristic="button", enable=True)
    while True:
        t.waitForNotifications(1.0)


@main.command()
@click.pass_context
def demo_led2(ctx):
    """
    Enable/Disable motion characteristics.
    """
    address = ctx.obj.get("address")

    t = thingy52.Thingy52(address)
    atexit.register(t.disconnect)
    t.setDelegate(delegates.ThingyCharDelegate(t.handles))
    t.ui.rgb_breathe(color=Color.CYAN)
    t.sound.play_sample()
    while True:
        t.waitForNotifications(1.0)
        print("Waiting...")



if __name__ == "__main__":
    main()



from bluepy.btle import DefaultDelegate
import math

from thingy52.services import AudioSample


class ThingyCharDelegate(DefaultDelegate):
    def __init__(self, handles):
        DefaultDelegate.__init__(self)
        self.handles = handles

    def handleNotification(self, cHandle, data):
        thingy_char = self.handles[cHandle]
        name = thingy_char.common_name

        print(name)
        print(len(["0x%02x" % b for b in data]), "BYTES")
        print(["0x%02x" % b for b in data])

        val = thingy_char.conversion_func(data)
        print("{}: {}".format(name, val))


class Demo1Delegate(DefaultDelegate):
    def __init__(self, thingy52, handles):
        DefaultDelegate.__init__(self)
        self.thingy52 = thingy52
        self.handles = handles
        self._button_activated = False

    def handleNotification(self, cHandle, data):
        thingy_char = self.handles[cHandle]
        name = thingy_char.common_name
        val = thingy_char.conversion_func(data)

        if name is "button" and val is True:
            self.on_button_pressed()
        elif name is "heading":
            r = abs(int(255 * math.sin(math.radians(val))))
            g = 0
            b = abs(int(255 * math.cos(math.radians(val))))
            self.thingy52.ui.rgb_constant(r, g, b)

    def on_button_pressed(self):
        self._button_activated = not self._button_activated
        if self._button_activated:
            self.thingy52.sound.play_sample(AudioSample.COIN_1)
            self.thingy52.ui.rgb_constant(0, 255, 0)
        else:
            self.thingy52.sound.play_sample(AudioSample.HIT)
            self.thingy52.ui.rgb_constant(255, 0, 0)
        self.thingy52.motion.toggle_notifications("heading", enable=self._button_activated)



from thingy52.uuids import *
from thingy52.characteristics import *
from enum import Enum
from random import randint


class Color(Enum):
    RED = 1
    GREEN = 2
    YELLOW = 3
    BLUE = 4
    PURPLE = 5
    CYAN = 6
    WHITE = 7


class LedMode(Enum):
    OFF = 0
    CONSTANT = 1
    BREATHE = 2
    ONE_SHOT = 3


class AudioSample(Enum):
    COIN_1 = 0
    COIN_2 = 1
    EXPLOSION_1 = 2
    EXPLOSION_2 = 3
    HIT = 4
    PICKUP_1 = 5
    PICKUP_2 = 6
    SHOOT_1 = 7
    SHOOT_2 = 8


class ThingyService(object):
    """
    Thing service base class
    """

    service_uid = None

    notify_chars = []

    write_chars = []

    def __init__(self, peripheral):
        self.peripheral = peripheral
        self.uuid = Nordic_UUID(self.service_uid)
        self.service = self.peripheral.getServiceByUUID(self.uuid)

        self.chars_map = {c.common_name: c for c in self.notify_chars}

    def list_notifications(self):
        return self.chars_map.keys()

    def toggle_notifications(self, characteristic=None, enable=True):
        """
        Switches notifications on or off.
        :param enable: notifications on or off
        :param characteristic:
        :return:
        """
        if characteristic:
            char_uuid = self.chars_map[characteristic].uuid
            char = self.service.getCharacteristics(forUUID=char_uuid)[0]
            self.peripheral.toggle_notify(char, enable)
        else:
            # get all characteristics for service
            for c in self.service.getCharacteristics():
                self.peripheral.toggle_notify(c, enable)


class EnvironmentService(ThingyService):
    """
    Environment service module. Instance the class and enable to get access to the Environment interface.
    """
    service_uid = ENVIRONMENT_SERVICE_UUID

    notify_chars = [
        ThingyChar(Nordic_UUID(E_TEMPERATURE_CHAR_UUID), 'temperature', temp),
        ThingyChar(Nordic_UUID(E_PRESSURE_CHAR_UUID), 'pressure', pressure),
        ThingyChar(Nordic_UUID(E_HUMIDITY_CHAR_UUID), 'humidity', humidity),
        ThingyChar(Nordic_UUID(E_GAS_CHAR_UUID), 'gas', extract_gas_data),
        ThingyChar(Nordic_UUID(E_COLOR_CHAR_UUID), 'color', b2a_hex),
        ThingyChar(Nordic_UUID(E_CONFIG_CHAR_UUID), 'config', b2a_hex),
    ]


class MotionService(ThingyService):
    """
    Environment service module. Instance the class and enable to get access to the Environment interface.
    """
    service_uid = MOTION_SERVICE_UUID

    notify_chars = [
        ThingyChar(Nordic_UUID(M_TAP_CHAR_UUID), 'tap', extract_tap_data),
        ThingyChar(Nordic_UUID(M_ORIENTATION_CHAR_UUID), 'orientation', c_type_double),
        ThingyChar(Nordic_UUID(M_QUATERNION_CHAR_UUID), 'quaternion', b2a_hex),
        ThingyChar(Nordic_UUID(M_STEP_COUNTER_UUID), 'step_cont', b2a_hex),
        ThingyChar(Nordic_UUID(M_RAW_DATA_CHAR_UUID), 'rawdata', b2a_hex),
        ThingyChar(Nordic_UUID(M_EULER_CHAR_UUID), 'euler', b2a_hex),
        ThingyChar(Nordic_UUID(M_ROTATION_MATRIX_CHAR_UUID), 'rotation', b2a_hex),
        ThingyChar(Nordic_UUID(M_HEAIDNG_CHAR_UUID), 'heading', b2a_hex),
        ThingyChar(Nordic_UUID(M_GRAVITY_VECTOR_CHAR_UUID), 'gravity', b2a_hex),
    ]


class UserInterfaceService(ThingyService):
    """
    Environment service module. Instance the class and enable to get access to the Environment interface.
    """
    service_uid = USER_INTERFACE_SERVICE_UUID

    notify_chars = [
        ThingyChar(Nordic_UUID(UI_BUTTON_CHAR_UUID), 'button', unpack_bool),
        ThingyChar(Nordic_UUID(UI_LED_CHAR_UUID), 'led', unpack_bool),
    ]

    """write_chars = [
        ThingyChar(Nordic_UUID(UI_LED_CHAR_UUID), 'led', unpack_bool),
    ]"""

    def rgb_off(self):
        char_uuid = self.chars_map["led"].uuid
        char = self.service.getCharacteristics(forUUID=char_uuid)[0]
        s = Struct('> 4B')
        led_command = s.pack(LedMode.CONSTANT.value)
        char.write(led_command, True)

    def rgb_constant(self, r, g, b):
        char_uuid = self.chars_map["led"].uuid
        char = self.service.getCharacteristics(forUUID=char_uuid)[0]

        s = Struct('> 4B')
        values = (LedMode.CONSTANT.value, r, g, b)
        led_command = s.pack(*values)
        char.write(led_command, True)

    def rgb_breathe(self, color=Color.RED, intensity=100, delay=1):
        char_uuid = self.chars_map["led"].uuid
        char = self.service.getCharacteristics(forUUID=char_uuid)[0]

        s = Struct('> 3BH')
        values = (LedMode.BREATHE.value, color.value, intensity, delay)
        led_command = s.pack(*values)
        char.write(led_command, True)

    def rgb_one_shot(self, color=Color.RED, intensity=100):
        char_uuid = self.chars_map["led"].uuid
        char = self.service.getCharacteristics(forUUID=char_uuid)[0]

        s = Struct('> 3B')
        values = (LedMode.ONE_SHOT.value, color.value, intensity)
        led_command = s.pack(*values)
        char.write(led_command, True)


class SoundService(ThingyService):
    """
    Environment service module. Instance the class and enable to get access to the Environment interface.
    """
    service_uid = SOUND_SERVICE_UUID

    notify_chars = [
        ThingyChar(Nordic_UUID(S_SPEAKER_STATUS_CHAR_UUID), 'speaker_status', b2a_hex),
        ThingyChar(Nordic_UUID(S_CONFIG_CHAR_UUID), 'config_characteristic', b2a_hex),
        ThingyChar(Nordic_UUID(S_SPEAKER_DATA_CHAR_UUID), 'speaker_characteristic', b2a_hex),
        ThingyChar(Nordic_UUID(S_MICROPHONE_CHAR_UUID), 'microphone', b2a_hex),
    ]

    def activate_speaker_stream(self, speaker_mode=2, mic_mode=1):
        char_uuid_config = self.chars_map["config_characteristic"].uuid
        char_config = self.service.getCharacteristics(forUUID=char_uuid_config)[0]
        bytes = Struct('> 2B').pack(speaker_mode, mic_mode)
        char_config.write(bytes, True)

    def stream_speaker(self):
        char_uuid = self.chars_map["speaker_characteristic"].uuid
        char = self.service.getCharacteristics(forUUID=char_uuid)[0]
        s = Struct('> 120B')
        m = (randint(0, 100) for i in range(120))
        command = s.pack(*m)
        print("packed")
        char.write(command, True)
        print("written")

    def stream_frequency(self, frequency, duration, volume):
        char_uuid = self.chars_map["speaker_characteristic"].uuid
        char = self.service.getCharacteristics(forUUID=char_uuid)[0]
        s = Struct('> HHB')
        command = s.pack(frequency, duration, volume)
        char.write(command, True)

    def play_sample(self, sample=AudioSample.SHOOT_1):
        char_uuid_config = self.chars_map["config_characteristic"].uuid
        char_config = self.service.getCharacteristics(forUUID=char_uuid_config)[0]
        bytes = Struct('> 2B').pack(3, 1)
        char_config.write(bytes, True)

        char_uuid = self.chars_map["speaker_characteristic"].uuid
        char = self.service.getCharacteristics(forUUID=char_uuid)[0]

        s = Struct('> B')
        command = s.pack(sample.value)
        char.write(command, True)



from bluepy.btle import UUID

# Please see # Ref https://nordicsemiconductor.github.io/Nordic-Thingy52-FW/documentation
# for more information on the UUIDs of the Services and Characteristics that are being used
def Nordic_UUID(val):
    """ Adds base UUID and inserts value to return Nordic UUID """
    return UUID("EF68%04X-9B35-4933-9B10-52FFA9740042" % val)


# Definition of all UUID used by Thingy
CCCD_UUID = 0x2902

BATTERY_SERVICE_UUID = 0x180F
BATTERY_LEVEL_UUID = 0x2A19

ENVIRONMENT_SERVICE_UUID = 0x0200
E_TEMPERATURE_CHAR_UUID = 0x0201
E_PRESSURE_CHAR_UUID    = 0x0202
E_HUMIDITY_CHAR_UUID    = 0x0203
E_GAS_CHAR_UUID         = 0x0204
E_COLOR_CHAR_UUID       = 0x0205
E_CONFIG_CHAR_UUID      = 0x0206

USER_INTERFACE_SERVICE_UUID = 0x0300
UI_LED_CHAR_UUID            = 0x0301
UI_BUTTON_CHAR_UUID         = 0x0302
UI_EXT_PIN_CHAR_UUID        = 0x0303

MOTION_SERVICE_UUID         = 0x0400
M_CONFIG_CHAR_UUID          = 0x0401
M_TAP_CHAR_UUID             = 0x0402
M_ORIENTATION_CHAR_UUID     = 0x0403
M_QUATERNION_CHAR_UUID      = 0x0404
M_STEP_COUNTER_UUID         = 0x0405
M_RAW_DATA_CHAR_UUID        = 0x0406
M_EULER_CHAR_UUID           = 0x0407
M_ROTATION_MATRIX_CHAR_UUID = 0x0408
M_HEAIDNG_CHAR_UUID         = 0x0409
M_GRAVITY_VECTOR_CHAR_UUID  = 0x040A

SOUND_SERVICE_UUID          = 0x0500
S_CONFIG_CHAR_UUID          = 0x0501
S_SPEAKER_DATA_CHAR_UUID    = 0x0502
S_SPEAKER_STATUS_CHAR_UUID  = 0x0503
S_MICROPHONE_CHAR_UUID      = 0x0504




from thingy52.thingy52 import Thingy52
from thingy52.delegates import ThingyCharDelegate
import atexit
t = Thingy52("E2:D9:D5:C6:30:26")


atexit.register(t.disconnect)
t.setDelegate(ThingyCharDelegate(t.handles))
t.environment.toggle_notifications(characteristic="temperature", enable=True)
while True:
    t.waitForNotifications(1.0)
    print("Waiting...")



import wave
import sys
import audioop
import binascii

wave_file_path = "audiop_output.wav"
# Get the fragments of ADPCM data:
# Here, blocks is an array of 259byte data represented as strings which is parsed
# from the ADPCM data file, write this according to the structure of your ADPCM file
block = getAdpcmFragments(adpcm_file_path)

#Set parameters for the wavefile according to your audio stream
wave_file = wave.open(wave_file_path, 'wb')
wave_file.setparams((2,2,16000, 0, 'NONE', 'NONE'))

# Using ASCII to Binary from the binascii lib to convert strings represented as e.g
# 4A, 6F etc into actual binary values. Then I am writing the linear PCM data from
# adpcm2lin to the wav-file.
state = None
for this in block:
pcm, state = audioop.adpcm2lin(binascii.a2b_hex(this), 4, state)
wave_file.writeframes(pcm)



from collections import namedtuple
from struct import Struct
from binascii import b2a_hex

from thingy52.uuids import *

ThingyChar = namedtuple('ThingyChar', ['uuid', 'common_name', 'conversion_func'])

c_type_double = Struct('< d').unpack


def unpack_float(integer, decimal):
    return float(integer + decimal / 100)


def unpack_bool(data):
    val = Struct('< B').unpack(data)
    return bool(val[0])


def temp(data):
    s = Struct('< bB')  # < (little endian) B (unsigned int single byte)
    (integer, decimal) = s.unpack(data)
    temperature = unpack_float(integer, decimal)
    return temperature


def pressure(data):
    s = Struct('< iB')  # < (little endian) B (integer single byte)
    (integer, decimal) = s.unpack(data)
    return unpack_float(integer, decimal)


def humidity(data):
    s = Struct('< B')
    return s.unpack(data)


def extract_gas_data(data):
    """ Extract gas data from data string. """
    teptep = b2a_hex(data)
    eco2 = int(teptep[:2]) + (int(teptep[2:4]) << 8)
    tvoc = int(teptep[4:6]) + (int(teptep[6:8]) << 8)
    return eco2, tvoc


def extract_tap_data(data):
    """ Extract tap data from data string. """
    teptep = b2a_hex(data)
    direction = teptep[0:2]
    count = teptep[2:4]
    return (direction, count)

def rotation(data):
    """
    3 rotation axes
    :param data:
    :return:
    """
    s = Struct('< 9h')
    rot_matrix = s.unpack(data)
    return rot_matrix

def euler(data):
    (roll, pitch, yaw) = Struct('< 3i').unpack(data)
    return (roll, pitch, yaw)

def degToCompass(num):
    val=int((num/22.5)+.5)
    arr=["N","NNE","NE","ENE","E","ESE", "SE", "SSE","S","SSW","SW","WSW","W","WNW","NW","NNW"]
    return arr[(val % 16)]

def heading_degrees(data):
    h = Struct('< i').unpack(data)
    heading = int(360 * h[0] / 23580000)
    return heading

def heading(data):
    h = Struct('< i').unpack(data)
    heading = int(360 * h[0] / 23580000)
    compass = degToCompass(heading)
    return compass

def orientation(data):
    h = int(Struct('< B').unpack(data)[0])
    return h


battery_level = ThingyChar(Nordic_UUID(BATTERY_LEVEL_UUID), 'battery level', b2a_hex)
temperature = ThingyChar(Nordic_UUID(E_TEMPERATURE_CHAR_UUID), 'temperature', temp)
pressure = ThingyChar(Nordic_UUID(E_PRESSURE_CHAR_UUID), 'pressure', pressure)
humidity = ThingyChar(Nordic_UUID(E_HUMIDITY_CHAR_UUID), 'humidity', humidity)
gas = ThingyChar(Nordic_UUID(E_GAS_CHAR_UUID), 'gas', extract_gas_data)
color = ThingyChar(Nordic_UUID(E_COLOR_CHAR_UUID), 'color', b2a_hex)
config = ThingyChar(Nordic_UUID(E_CONFIG_CHAR_UUID), 'config', b2a_hex)

ui_button = ThingyChar(Nordic_UUID(UI_BUTTON_CHAR_UUID), 'button', unpack_bool)
ui_led = ThingyChar(Nordic_UUID(UI_LED_CHAR_UUID), 'led', unpack_bool)

m_tap = ThingyChar(Nordic_UUID(M_TAP_CHAR_UUID), 'tap', extract_tap_data)
m_tap = ThingyChar(Nordic_UUID(M_TAP_CHAR_UUID), 'tap', extract_tap_data)
m_orient = ThingyChar(Nordic_UUID(M_ORIENTATION_CHAR_UUID), 'orientation', orientation)
m_quaternion = ThingyChar(Nordic_UUID(M_QUATERNION_CHAR_UUID), 'quaternion', b2a_hex)
m_stepcnt = ThingyChar(Nordic_UUID(M_STEP_COUNTER_UUID), 'step_count', b2a_hex)
m_rawdata = ThingyChar(Nordic_UUID(M_RAW_DATA_CHAR_UUID), 'rawdata', b2a_hex)
m_euler = ThingyChar(Nordic_UUID(M_EULER_CHAR_UUID), 'euler', euler)
m_rotation = ThingyChar(Nordic_UUID(M_ROTATION_MATRIX_CHAR_UUID), 'rotation', rotation)
m_heading = ThingyChar(Nordic_UUID(M_HEAIDNG_CHAR_UUID), 'heading', heading_degrees)
m_gravity = ThingyChar(Nordic_UUID(M_GRAVITY_VECTOR_CHAR_UUID), 'gravity', b2a_hex)

s_speaker_status = ThingyChar(Nordic_UUID(S_SPEAKER_STATUS_CHAR_UUID), 'speaker_status', b2a_hex)
config_characteristic = ThingyChar(Nordic_UUID(S_CONFIG_CHAR_UUID), 'config_characteristic', b2a_hex)
speaker_characteristic = ThingyChar(Nordic_UUID(S_SPEAKER_DATA_CHAR_UUID), 'speaker_characteristic', b2a_hex)
s_microphone = ThingyChar(Nordic_UUID(S_MICROPHONE_CHAR_UUID), 'microphone', lambda bytes: bytes)

thingy_chars = [
    temperature,
    pressure,
    humidity,
    gas,
    color,
    ui_button,
    ui_led,
    m_tap,
    m_orient,
    m_quaternion,
    m_stepcnt,
    m_rawdata,
    m_euler,
    m_rotation,
    m_heading,
    m_gravity,
    s_speaker_status,
    config_characteristic,
    speaker_characteristic,
    s_microphone,
]

thingy_chars_map = {c.uuid: c for c in thingy_chars}
